
#import "SplitStartController.h"

#import "StartController.h"
#import "WebController.h"

static const CGFloat kBorderWidth = 1;

///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
@interface SplitStartController()

- (void)setupNavigators;

@end



///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
@implementation SplitStartController


///////////////////////////////////////////////////////////////////////////////////////////////////
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
  if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
    [self setupNavigators];
  }
  
  return self;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (void)updateLayoutWithOrientation:(UIInterfaceOrientation)interfaceOrientation {
  [super updateLayoutWithOrientation:interfaceOrientation];
  
  _dividerView.height = self.view.height;
  _dividerView.width = kBorderWidth;
  _dividerView.right = self.primaryViewController.view.left + _dividerView.width;
  _dividerView.top = self.primaryViewController.view.top;
  
  [self.view bringSubviewToFront:_dividerView];
}


///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
#pragma mark -
#pragma mark UIViewController


///////////////////////////////////////////////////////////////////////////////////////////////////
- (void)loadView {
  [super loadView];
  
  self.view.autoresizingMask = (UIViewAutoresizingFlexibleWidth
                                | UIViewAutoresizingFlexibleHeight);
  
  _dividerView = [[UIView alloc] init];
  _dividerView.backgroundColor = [UIColor lightGrayColor];
  
  _dividerView.autoresizingMask = (UIViewAutoresizingFlexibleHeight);
  [self.view addSubview:_dividerView];
  
  [self updateLayoutWithOrientation:TTInterfaceOrientation()];
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (void)viewDidUnload {
  [super viewDidUnload];
  
  TT_RELEASE_SAFELY(_dividerView);
}


///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
#pragma mark -
#pragma mark TTSplitViewController


///////////////////////////////////////////////////////////////////////////////////////////////////
- (void)primaryViewDidAppear:(BOOL)animated {
  [super primaryViewDidAppear:animated];
  
  [self.view bringSubviewToFront:_dividerView];
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (void)openSecondaryURLAction:(NSURL*)url {
  self.primaryNavigator.rootViewController = nil;
  [self.primaryNavigator openURLAction:[TTURLAction actionWithURLPath:url.absoluteString]];
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (void)setupSecondaryNavigator {
  TTURLMap* map = self.secondaryNavigator.URLMap;
  
  // Forward all unhandled URL actions to the right navigator.
  [map              from: @"*"
                toObject: self
                selector: @selector(openSecondaryURLAction:)];
  
  [map                    from: kAppStartURLPath
              toViewController: [StartController class]];
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (void)setupPrimaryNavigator {
  TTURLMap* map = self.primaryNavigator.URLMap;
  
  [map                    from: @"*"
              toViewController: [WebController class]];
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (void)setupNavigators {
  [self setupPrimaryNavigator];
  [self setupSecondaryNavigator];
  
  [self.secondaryNavigator openURLs:kAppStartURLPath, nil];
  [self.primaryNavigator openURLs:@"", nil];
}

@end

